var SMALL_BTN_SIZE = 25;
var SMALL_BTN_MARGIN = 5;
var TOO_EXPENSIVE_ALPHA = 0.4;
var BOUGHT_MARK = new Sprite("../img/tickmark.png");
var _globalUBId = 0;

function UpgradeButton(position, size, background, doneBgColour, object, price, textSize, textColour, font, borderColour, borderWidth, popOverData) {
    "use strict";

    var buttonId = _globalUBId++;
    var text = object;

    if (price > 0) {
        this.price = price;
        text = "Buy " + object + " ($" + price + ")";
    }

    this.object = object;
    this.bought = false;
    this.upgrades = [];
    this.doneBgColour = doneBgColour;
    this.isVisible = false;

    if (popOverData) {
        this.popOver = new PopOver(popOverData.size, popOverData.title, popOverData.titleSize, popOverData.price, popOverData.text, popOverData.textSize, popOverData.bgColour, popOverData.textColour);
    }

    if (typeof position === 'undefined') {
        position = new Vector2(SMALL_BTN_MARGIN * 1.5, 0);
        this.isSmall = true;
    }

    if (typeof size === 'undefined') {
        size = new Vector2(SMALL_BTN_SIZE, SMALL_BTN_SIZE);
    }

    var origBgColour = background;

    this.btn = new Button(position, size, background, text, textSize, textColour, font, borderColour, borderWidth);

    var self = this;

    this.addUpgrade = function (upgrade) {
        upgrade.btn.position.add(this.btn.position.copy().add(
            new Vector2(
                this.btn.getCollisionWidth(), //
                (this.btn.getCollisionHeight() - upgrade.btn.getCollisionHeight()) / 2
            )));

        upgrade.btn.position.add(
            new Vector2(
                (SMALL_BTN_MARGIN + SMALL_BTN_SIZE) * self.upgrades.length,
                0
            )
        );

        self.upgrades.push(upgrade);
    };

    this.btn.onmouseover = function (mouse) {
        if (self.popOver && self.isVisible && (self.isSmall || !self.bought)) {
            self.popOver.isVisible = true;
        }
    };

    this.reset = function () {
        this.bought = false;
        this.isVisible = false;
        this.btn.bgColour = origBgColour;
        for (var upgrade of this.upgrades) {
            upgrade.reset();
        }
    };

    this.loadSaveData = function (cookieManager) {
        var data = cookieManager.getObject("UB" + buttonId + "_data");
        
        this.bought = data.substr(0, 1) === 'T';
        this.isVisible = data.substr(1, 1) === 'T';
        this.btn.bgColour = data.substr(2);

        for (var upgrade of this.upgrades) {
            upgrade.loadSaveData(cookieManager);
        }
    }

    this.getSaveData = function () {
        var data = [];

        data.push({
            name: "UB" + buttonId + "_data",
            value: (this.bought ? 'T' : 'F') + (this.isVisible ? 'T' : 'F') + (this.btn.bgColour)
        });

        for (var upgrade of this.upgrades) {
            data = data.concat(upgrade.getSaveData());
        }

        return data;
    }
}

UpgradeButton.prototype.toString = function () {
    return "UpgradeButton { " + this.btn.text + " }";
};

UpgradeButton.prototype.buy = function (status, forceBuy) {
    if (this.bought) {
        Log.error("You already own this, silly!", this);
    } else if (!this.isVisible) {
        Log.error("How Can You Buy This If It Isn't Even Real?!", this);
    } else if (status.money < this.price) {
        Log.error("That's too expensive! Money: " + status.money //
            + ", Price: " + this.price, this)
    } else {
        if (!forceBuy) {
            status.money -= this.price;
        }
        this.bought = true;
        if (this.doneBgColour) {
            this.btn.bgColour = this.doneBgColour;
        }
        if (RENDER_DEBUG_TEXT) {
            Log.debug("Succesfully bought " + this.object, this);
        }
        return true;
    }

    return false;
};

UpgradeButton.prototype.render = function (screen, status, alpha) {

    // Calculate the alpha value.
    if (status && typeof alpha !== 'number') {
        alpha = status.money >= this.price ? 1.0 : TOO_EXPENSIVE_ALPHA;
    }

    var bgShadow = "#000";

    if (this.bought) {
        alpha = 1.0;
    } else if (alpha < 0.9) {
        bgShadow = undefined;
    }

    if (alpha > 0) {
        this.isVisible = true;
    }

    this.btn.render(screen, alpha, "#000", bgShadow);

    // Render tickmark on small upgrades that have been bought.
    if (this.bought && this.isSmall) {
        var pos = this.btn.position.copy().add(
            new Vector2(this.btn.borderWidth / 2, this.btn.borderWidth / 2));
        BOUGHT_MARK.render(screen, pos);
    }

    if (this.popOver && this.popOver.isVisible) {
        this.popOver.render(screen, this.btn.position.copy().add(new Vector2(this.btn.getCollisionWidth() / 2, 0)));

        if (!this.btn.containsPoint(ojglMouse.position)) {
            this.popOver.isVisible = false;
        }
    }

    // Render available small upgrades.
    if (alpha >= 1.0 && this.bought) {
        renderUpgrades: //
            for (var upgrade of this.upgrades) {
            upgrade.render(screen, status,
                this.bought ? undefined : TOO_EXPENSIVE_ALPHA);
            if (!upgrade.bought) {
                break renderUpgrades;
            }
        }
    }
};

UpgradeButton.prototype.willAppear = function () {
    this.btn.willAppear();
    for (var upgrade of this.upgrades) {
        upgrade.willAppear();
    }
};

UpgradeButton.prototype.willDisappear = function () {
    this.btn.willDisappear();
    for (var upgrade of this.upgrades) {
        upgrade.willDisappear();
    }
};